'''
Simple Star/Galaxy Binary Classification in Pytorch
Network Architecture
    
Author: Nima Sedaghat

BigSkyEarth ُTraining School
Tuorla Observatory, Finland
November 2018
'''

import torch
from torch.nn import *

class sgClassifier(torch.nn.Module):                                                                                                

    def __init__(self):

        super().__init__()
        
        #--- Define layers
        self.Conv1 = Conv2d(5, 64, kernel_size=(3, 3), stride=(1, 1))
        self.ReLU1 = ReLU()
        self.Pool1 = MaxPool2d(kernel_size=2, stride=2)

        self.Conv2 = Conv2d(64, 128, kernel_size=(3, 3), stride=(1, 1))
        self.ReLU2 = ReLU()
        self.Pool2 = MaxPool2d(kernel_size=2, stride=2)

        self.Conv3 = Conv2d(128, 256, kernel_size=(3, 3), stride=(1, 1))
        self.ReLU3 = ReLU()
        self.Pool3 = MaxPool2d(kernel_size=2, stride=2)

        self.FC4 = Linear(in_features=4096, out_features=512)
        self.ReLU4 = ReLU()
        self.Drop4 = Dropout(p=0)

        self.FC5 = Linear(in_features=512, out_features=512)
        self.ReLU5 = ReLU()
        self.Drop5 = Dropout(p=0)

        self.FC6 = Linear(in_features=512, out_features=2)


    def forward(self,x):
        B = x.shape[0] #obtain the batch size 

        x1 = self.Pool1.forward(self.ReLU1.forward(self.Conv1.forward(x)))
        x2 = self.Pool2.forward(self.ReLU2.forward(self.Conv2.forward(x1)))
        x3 = self.Pool3.forward(self.ReLU3.forward(self.Conv3.forward(x2)))
        #x3_ = x3.reshape(B,-1)
        x3_ = x3.view(B,-1) 
        x4 = self.Drop4(self.ReLU4.forward(self.FC4.forward(x3_)))
        x5 = self.Drop5(self.ReLU5.forward(self.FC5.forward(x4)))
        output = self.FC6.forward(x5)

        return output
