'''
Simple Star/Galaxy Binary Classification in Pytorch
    
Author: Nima Sedaghat

BigSkyEarth ُTraining School
Tuorla Observatory, Finland
November 2018
'''

import numpy as np
import torch
import torchvision.models as models

from model import sgClassifier

#--- Create the model
model = sgClassifier()

#--- Load data (downloaded from https://github.com/EdwardJKim/dl4astro.git)
data = np.load('data/dl4astro/data/sdss_training_images.npy')
labels = np.load('data/dl4astro/data/sdss_training_labels.npy')
labels = ((labels+1)/2).astype(int) # Convert labels to the required format (we want 0/1, not -1/+1)

# load validation data too
vdata = np.load('data/dl4astro/data/sdss_test_images.npy')
vlabels = np.load('data/dl4astro/data/sdss_test_labels.npy')
vlabels = ((vlabels+1)/2).astype(int)

#--- Set up the solver
learning_rate = 1e-4
optimizer = torch.optim.Adam(model.parameters(), lr=learning_rate)
scheduler = torch.optim.lr_scheduler.MultiStepLR(optimizer, milestones=[])

#--- Training loop
N = data.shape[0]
batch_size = 100
CE = torch.nn.CrossEntropyLoss();

LOSS = []
VLOSS = []
for epoch in range(2000):
    print('Epoch:',epoch)

    model.train()

    scheduler.step()
    
    for i in range(0,N,batch_size):

        batch = data[i:min(N,i+batch_size)];
        lbatch = labels[i:min(N,i+batch_size)]; 

        batch_tensor = torch.from_numpy(batch)
        lbatch_tensor = torch.from_numpy(lbatch)

        #--- Forward pass
        output = model(batch_tensor)

        #--- Compute loss
        loss = CE(output,lbatch_tensor)

        #--- and accuracy
        binarized = output.max(dim=1)[1]
        accuracy = (binarized == lbatch_tensor).nonzero().numel() / batch_size

        #--- Pytorch backward+update
        optimizer.zero_grad()
        loss.backward()
        optimizer.step()

        print('loss:',loss.data)#,accuracy)

    #--- Validation
    model.eval()
    voutput = model(torch.from_numpy(vdata))
    vloss = CE(voutput,torch.from_numpy(vlabels))
    print('validation loss:',vloss.data)

    LOSS.append(loss.detach().cpu().numpy())
    VLOSS.append(vloss.detach().cpu().numpy())


import matplotlib.pyplot as plt
plt.plot(LOSS)
plt.plot(VLOSS,'r')
plt.show(block=True)

